@@ -38,6 +38,12 @@ class KoStore;
 class KoOdfWriteStore;
 class KoOdfReadStore;
 
+#define USERFIELD_TRANSLATION -3
+#define USERFIELD_VARIABLE -2
+#define USERFIELD_NONE -1
+#define USERFIELD_HEADER 0
+#define USERFIELD_DATA 1
+
 namespace KPlato
 {
 
@@ -46,16 +52,16 @@ class ScheduleManager;
 class ItemModelBase;
 
 
-class KPLATOMODELS_EXPORT ReportGeneratorOdt : public ReportGenerator
+class PLANMODELS_EXPORT ReportGeneratorOdt : public ReportGenerator
 {
 public:
     explicit ReportGeneratorOdt();
-    ~ReportGeneratorOdt();
+    ~ReportGeneratorOdt() override;
 
-    bool open();
-    void close();
+    bool open() override;
+    void close() override;
 
-    bool createReport();
+    bool createReport() override;
 
 protected:
     bool createReportOdt();
@@ -82,14 +88,17 @@ protected:
     void handleUserFieldDecls(KoXmlWriter &writer, const KoXmlElement &decls);
     QAbstractItemModel *dataModel(const QString &name) const;
 
+    void addDataModel(const QString &name, QAbstractItemModel *model, int role);
+
 public:
     class UserField {
     public:
         enum Variant { None, Header, Rows, Variable, Translation };
-        UserField() : seqNr(-1), serieNr(0) {}
+        UserField() : seqNr(USERFIELD_NONE), serieNr(0) {}
         int variant() const;
         void begin();
         bool next();
+        void finish();
         QString headerData(const QString &columnName) const;
         QString data(const QString &columnName) const;
 
@@ -106,7 +115,7 @@ public:
         QStringList columns; // Lowercase list of headernames that shall be used
         QSortFilterProxyModel model;
         int seqNr; // A sequence number used to tabulate column names (eg: seqNr=2: table1.name2)
-        int serieNr; // A chart needs to know which data serie it works with
+        int serieNr; // A chart needs to know which data series it works with
         QString hasLabels; // A chart needs to know if data shall contain legends/labels
 
         QModelIndex currentIndex;
