@@ -20,7 +20,7 @@
 #ifndef KPTWORKPACKAGESENDDIALOG_H
 #define KPTWORKPACKAGESENDDIALOG_H
 
-#include "kplatoui_export.h"
+#include "planui_export.h"
 
 #include <KoDialog.h>
 
@@ -32,14 +32,16 @@ class WorkPackageSendPanel;
 class Node;
 class ScheduleManager;
 
-class KPLATOUI_EXPORT WorkPackageSendDialog : public KoDialog
+class PLANUI_EXPORT WorkPackageSendDialog : public KoDialog
 {
     Q_OBJECT
 public:
-    explicit WorkPackageSendDialog( const QList<Node*> &tasks, ScheduleManager *sm, QWidget *parent=0);
+    explicit WorkPackageSendDialog(const QList<Node*> &tasks, ScheduleManager *sm, QWidget *parent=0);
 
     WorkPackageSendPanel *panel() const { return m_wp; }
 
+    QSize sizeHint() const override;
+
 private:
     WorkPackageSendPanel *m_wp;
 };
